@@ -2,7 +2,6 @@ from abc import ABC, abstractmethod
 from enum import Enum
 from typing import Any
 
-import numpy as np
 from numpy.random import Generator
 
 from phylogenie.skyline import SkylineParameterLike, skyline_parameter
@@ -32,8 +31,6 @@ class Event(ABC):
     def get_propensity(self, model: Model, time: float) -> float:
         n_individuals = model.count_individuals(self.state)
         rate = self.rate.get_value_at_time(time)
-        if rate == np.inf and not n_individuals:
-            return 0
         return rate * n_individuals
 
     @abstractmethod
