@@ -15,6 +15,6 @@ OneOrMany2DScalars = OneOrMany2D[Scalar]
 Many2DScalars = Many2D[Scalar]
 Many3DScalars = Many3D[Scalar]
 
-Vector1D = list[Scalar]
-Vector2D = list[Vector1D]
-Vector3D = list[Vector2D]
+Vector1D = tuple[Scalar, ...]
+Vector2D = tuple[Vector1D, ...]
+Vector3D = tuple[Vector2D, ...]
